.center-text {
    text-align: center;
}

label {
    display: -webkit-inline-box;
    font-weight: 300;
}

#input,
#output {
    width: 100%;
    height: 25%;
}

.export-buttons {
    display: inline;
    margin: 1rem;
}

#title {
    margin-top: 10px;
    margin-left: 5vw;
}

.ht_master .wtHolder {
    overflow-x: hidden;
}

.mg-top, h4 {
    margin-top: 40px;
}

.hot-container, .settings, .abbs-container {
    display: none;
}

.hot-container.active, .settings.active, .abbs-container.active {
    display: inline;
}